@@ -36,7 +36,8 @@ from pyrogram.api.errors import (
     PhoneMigrate, NetworkMigrate, PhoneNumberInvalid,
     PhoneNumberUnoccupied, PhoneCodeInvalid, PhoneCodeHashEmpty,
     PhoneCodeExpired, PhoneCodeEmpty, SessionPasswordNeeded,
-    PasswordHashInvalid, FloodWait, PeerIdInvalid, FilePartMissing
+    PasswordHashInvalid, FloodWait, PeerIdInvalid, FilePartMissing,
+    ChatAdminRequired
 )
 from pyrogram.api.types import (
     User, Chat, Channel,
@@ -52,6 +53,7 @@ from pyrogram.session import Auth, Session
 log = logging.getLogger(__name__)
 
 Config = namedtuple("Config", ["api_id", "api_hash"])
+Proxy = namedtuple("Proxy", ["enabled", "hostname", "port", "username", "password"])
 
 
 class Client:
@@ -65,7 +67,7 @@ class Client:
             to save the session to a file named ``<session_name>.session``.
 
         test_mode (:obj:`bool`, optional):
-            Enable or disable log-in to testing servers. Defaults to ``False``.
+            Enable or disable log-in to testing servers. Defaults to False.
             Only applicable for new sessions and will be ignored in case previously
             created sessions are loaded.
     """
@@ -89,6 +91,7 @@ class Client:
         self.markdown = Markdown(self.peers_by_id)
 
         self.config = None
+        self.proxy = None
         self.session = None
 
         self.update_handler = None
@@ -100,7 +103,7 @@ class Client:
         self.load_config()
         self.load_session(self.session_name)
 
-        self.session = Session(self.dc_id, self.test_mode, self.auth_key, self.config.api_id)
+        self.session = Session(self.dc_id, self.test_mode, self.proxy, self.auth_key, self.config.api_id)
 
         terms = self.session.start()
 
@@ -110,9 +113,9 @@ class Client:
             self.user_id = self.authorize()
             self.save_session()
 
-        self.session.update_handler = self.update_handler
         self.rnd_id = self.session.msg_id
         self.get_dialogs()
+        self.session.update_handler = self.update_handler
 
         mimetypes.init()
 
@@ -133,7 +136,7 @@ class Client:
         Args:
             stop_signals (:obj:`tuple`, optional):
                 Iterable containing signals the signal handler will listen to.
-                Defaults to ``(SIGINT, SIGTERM, SIGABRT)``.
+                Defaults to (SIGINT, SIGTERM, SIGABRT).
         """
         for s in stop_signals:
             signal(s, self.signal_handler)
@@ -151,7 +154,7 @@ class Client:
         self.update_handler = callback
 
     def send(self, data: Object):
-        """Use this method to send `Raw Function`_ queries.
+        """Use this method to send :ref:`Raw Function <using-raw-functions>` queries.
 
         This method makes possible to manually call every single Telegram API method in a low-level manner.
         Available functions are listed in the :obj:`pyrogram.api.functions` package and may accept compound
@@ -163,8 +166,6 @@ class Client:
 
         Raises:
             :class:`pyrogram.Error`
-
-        .. _`Raw Function`: https://github.com/pyrogram/pyrogram/wiki/Usage#using-raw-functions
         """
         return self.session.send(data)
 
@@ -192,9 +193,9 @@ class Client:
                 self.session.stop()
 
                 self.dc_id = e.x
-                self.auth_key = Auth(self.dc_id, self.test_mode).create()
+                self.auth_key = Auth(self.dc_id, self.test_mode, self.proxy).create()
 
-                self.session = Session(self.dc_id, self.test_mode, self.auth_key, self.config.api_id)
+                self.session = Session(self.dc_id, self.test_mode, self.proxy, self.auth_key, self.config.api_id)
                 self.session.start()
 
                 r = self.send(
@@ -291,21 +292,30 @@ class Client:
         return r.user.id
 
     def load_config(self):
-        config = ConfigParser()
-        config.read("config.ini")
+        parser = ConfigParser()
+        parser.read("config.ini")
 
         self.config = Config(
-            int(config["pyrogram"]["api_id"]),
-            config["pyrogram"]["api_hash"]
+            api_id=parser.getint("pyrogram", "api_id"),
+            api_hash=parser.get("pyrogram", "api_hash")
         )
 
+        if parser.has_section("proxy"):
+            self.proxy = Proxy(
+                enabled=parser.getboolean("proxy", "enabled"),
+                hostname=parser.get("proxy", "hostname"),
+                port=parser.getint("proxy", "port"),
+                username=parser.get("proxy", "username", fallback=None) or None,
+                password=parser.get("proxy", "password", fallback=None) or None
+            )
+
     def load_session(self, session_name):
         try:
             with open("{}.session".format(session_name)) as f:
                 s = json.load(f)
         except FileNotFoundError:
             self.dc_id = 1
-            self.auth_key = Auth(self.dc_id, self.test_mode).create()
+            self.auth_key = Auth(self.dc_id, self.test_mode, self.proxy).create()
         else:
             self.dc_id = s["dc_id"]
             self.test_mode = s["test_mode"]
@@ -1239,6 +1249,7 @@ class Client:
         )
 
     def delete_messages(self,
+                        chat_id: int or str,
                         message_ids: list,
                         revoke: bool = None):
         """Use this method to delete messages, including service messages, with the following limitations:
@@ -1250,24 +1261,41 @@ class Client:
         - If the user has *can_delete_messages* permission in a supergroup or a channel, it can delete any message there.
 
         Args:
+            chat_id (:obj:`int` | :obj:`str`):
+                Unique identifier for the target chat or username of the target channel/supergroup
+                (in the format @username). For your personal cloud storage (Saved Messages) you can
+                simply use "me" or "self".
+
             message_ids (:obj:`list`):
                 List of identifiers of the messages to delete.
 
             revoke (:obj:`bool`, optional):
-                Deletes messages on both parts (for private chats).
+                Deletes messages on both parts.
+                This is only for private cloud chats and normal groups, messages on
+                channels and supergroups are always revoked (i.e.: deleted for everyone).
 
         Raises:
             :class:`pyrogram.Error`
         """
-        # TODO: Maybe "revoke" is superfluous.
-        # If I want to delete a message, chances are I want it to
-        # be deleted even from the other side
-        return self.send(
-            functions.messages.DeleteMessages(
-                id=message_ids,
-                revoke=revoke or None
+        peer = self.resolve_peer(chat_id)
+
+        if isinstance(peer, types.InputPeerChannel):
+            return self.send(
+                functions.channels.DeleteMessages(
+                    channel=peer,
+                    id=message_ids
+                )
+            )
+        else:
+            # TODO: Maybe "revoke" is superfluous.
+            # If I want to delete a message, chances are I want it to
+            # be deleted even from the other side
+            return self.send(
+                functions.messages.DeleteMessages(
+                    id=message_ids,
+                    revoke=revoke or None
+                )
             )
-        )
 
     # TODO: Remove redundant code
     def save_file(self, path: str, file_id: int = None, file_part: int = 0):
@@ -1280,7 +1308,7 @@ class Client:
         file_id = file_id or self.rnd_id()
         md5_sum = md5() if not is_big and not is_missing_part else None
 
-        session = Session(self.dc_id, self.test_mode, self.auth_key, self.config.api_id)
+        session = Session(self.dc_id, self.test_mode, self.proxy, self.auth_key, self.config.api_id)
         session.start()
 
         try:
@@ -1345,7 +1373,8 @@ class Client:
             session = Session(
                 dc_id,
                 self.test_mode,
-                Auth(dc_id, self.test_mode).create(),
+                self.proxy,
+                Auth(dc_id, self.test_mode, self.proxy).create(),
                 self.config.api_id
             )
 
@@ -1361,6 +1390,7 @@ class Client:
             session = Session(
                 dc_id,
                 self.test_mode,
+                self.proxy,
                 self.auth_key,
                 self.config.api_id
             )
@@ -1416,7 +1446,8 @@ class Client:
                 cdn_session = Session(
                     r.dc_id,
                     self.test_mode,
-                    Auth(r.dc_id, self.test_mode).create(),
+                    self.proxy,
+                    Auth(r.dc_id, self.test_mode, self.proxy).create(),
                     self.config.api_id,
                     is_cdn=True
                 )
@@ -1545,7 +1576,7 @@ class Client:
 
             return r
 
-    def export_chat_invite_link(self, chat_id: int or str):
+    def export_chat_invite_link(self, chat_id: int or str, new: bool = False):
         """Use this method to export an invite link to a supergroup or a channel.
 
         The user must be an administrator in the chat for this to work and must have the appropriate admin rights.
@@ -1555,23 +1586,54 @@ class Client:
                 Unique identifier for the target chat or username of the target channel/supergroup
                 (in the format @username).
 
+            new (:obj:`bool`):
+                The previous link will be deactivated and a new link will be generated.
+                This is also used to create the invite link in case it doesn't exist yet.
+
         Returns:
             On success, the exported invite link as string is returned.
 
         Raises:
             :class:`pyrogram.Error`
+
+        Note:
+            If the returned link is a new one it may take a while for it to be activated.
         """
         peer = self.resolve_peer(chat_id)
 
         if isinstance(peer, types.InputPeerChat):
-            return self.send(
-                functions.messages.ExportChatInvite(
-                    chat_id=peer.chat_id
-                )
-            ).link
+            if new:
+                return self.send(
+                    functions.messages.ExportChatInvite(
+                        chat_id=peer.chat_id
+                    )
+                ).link
+            else:
+                chat_full = self.send(
+                    functions.messages.GetFullChat(
+                        chat_id=peer.chat_id
+                    )
+                ).full_chat  # type: types.ChatFull
+
+                if isinstance(chat_full.exported_invite, types.ChatInviteExported):
+                    return chat_full.exported_invite.link
+                else:
+                    raise ChatAdminRequired
         elif isinstance(peer, types.InputPeerChannel):
-            return self.send(
-                functions.channels.ExportInvite(
-                    channel=peer
-                )
-            ).link
+            if new:
+                return self.send(
+                    functions.channels.ExportInvite(
+                        channel=peer
+                    )
+                ).link
+            else:
+                channel_full = self.send(
+                    functions.channels.GetFullChannel(
+                        channel=peer
+                    )
+                ).full_chat  # type: types.ChannelFull
+
+                if isinstance(channel_full.exported_invite, types.ChatInviteExported):
+                    return channel_full.exported_invite.link
+                else:
+                    raise ChatAdminRequired
