@@ -68,7 +68,7 @@ class Session:
 
     notice_displayed = False
 
-    def __init__(self, dc_id: int, test_mode: bool, auth_key: bytes, api_id: str, is_cdn: bool = False):
+    def __init__(self, dc_id: int, test_mode: bool, proxy: type, auth_key: bytes, api_id: str, is_cdn: bool = False):
         if not Session.notice_displayed:
             print("Pyrogram v{}, {}".format(__version__, __copyright__))
             print("Licensed under the terms of the " + __license__, end="\n\n")
@@ -76,7 +76,7 @@ class Session:
 
         self.is_cdn = is_cdn
 
-        self.connection = Connection(DataCenter(dc_id, test_mode))
+        self.connection = Connection(DataCenter(dc_id, test_mode), proxy)
 
         self.api_id = api_id
 
